﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web.Http;

using TestCommon;

namespace TestServer
{
    /// <summary>
    /// 제품들 컨트롤러
    /// </summary>
    public class ProductsController : ApiController
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 제품 리스트
        /// </summary>
        private static List<Product> _productList = new List<Product> 
        { 
            new Product { ID = 1, Name = "Tomato Soup", Category = "Groceries", Price = 1      }, 
            new Product { ID = 2, Name = "Yo-yo"      , Category = "Toys"     , Price = 3.75M  }, 
            new Product { ID = 3, Name = "Hammer"     , Category = "Hardware" , Price = 16.99M } 
        };

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 제품 추가하기 - AddProduct(source)

        /// <summary>
        /// 제품 추가하기
        /// </summary>
        /// <param name="source">소스 제품</param>
        /// <returns>처리 결과</returns>
        [HttpPost]
        public IHttpActionResult AddProduct(Product source)
        {
            _productList.Add(source);

            return Ok("SUCCESS");
        }

        #endregion
        #region 제품 수정하기 - UpdateProduct(source)

        /// <summary>
        /// 제품 수정하기
        /// </summary>
        /// <param name="source">소스 제품</param>
        /// <returns>처리 결과</returns>
        [HttpPut]
        public IHttpActionResult UpdateProduct(Product source)
        {
            for(int i = 0; i < _productList.Count; i++)
            {
                Product product = _productList[i];

                if(product.ID == source.ID)
                {
                    _productList[i].Name     = source.Name;
                    _productList[i].Category = source.Category;
                    _productList[i].Price    = source.Price;

                    return Ok("SUCCESS");
                }
            }

            return Ok("NOUPDATE");
        }

        #endregion
        #region 제품 삭제하기 - DeleteProduct(id)

        /// <summary>
        /// 제품 삭제하기
        /// </summary>
        /// <param name="id">ID</param>
        /// <returns>처리 결과</returns>
        [HttpDelete]
        public IHttpActionResult DeleteProduct(int id)
        {
            for(int i = 0; i < _productList.Count; i++)
            {
                if(_productList[i].ID == id)
                {
                    _productList.RemoveAt(i);

                    return Ok("SUCCESS");
                }
            }

            return Ok("NODELETE");
        }

        #endregion

        #region 모든 제품 리스트 구하기 - GetAllProductList()

        /// <summary>
        /// 모든 제품 리스트 구하기
        /// </summary>
        /// <returns>모든 제품 리스트</returns>
        public List<Product> GetAllProductList()
        {
            return _productList;
        }

        #endregion
        #region 카테고리로 제품 리스트 구하기 - GetProductListByCategory(category)

        /// <summary>
        /// 카테고리로 제품 리스트 구하기
        /// </summary>
        /// <param name="category">카테고리</param>
        /// <returns>제품 리스트</returns>
        public List<Product> GetProductListByCategory(string category)
        {
            return _productList.Where(p => string.Equals(p.Category, category, StringComparison.OrdinalIgnoreCase)).ToList();
        }

        #endregion
        #region 제품 구하기 - GetProduct(id)

        /// <summary>
        /// 제품 구하기
        /// </summary>
        /// <param name="id">ID</param>
        /// <returns>제품</returns>
        public IHttpActionResult GetProduct(int id)
        {
            Product product = _productList.FirstOrDefault(p => p.ID == id);

            if(product == null)
            {
                return NotFound();
            }

            return Ok(product);
        }

        #endregion
    }
}