﻿using System;
using System.Diagnostics;
using System.Web.Http;
using System.Web.Http.SelfHost;

namespace TestServer
{
    /// <summary>
    /// 프로그램
    /// </summary>
    class Program
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region 프로그램 시작하기 - Main()

        /// <summary>
        /// 프로그램 시작하기
        /// </summary>
        private static void Main()
        {
            string serverURL      = string.Format("http://127.0.0.1:{0}", 8080);
            string bindingAddress = string.Format("http://+:{0}/"       , 8080);

            HttpSelfHostConfiguration configuration = new HttpSelfHostConfiguration(serverURL);

            configuration.Routes.MapHttpRoute("API Default", "api/{controller}/{id}", new { id = RouteParameter.Optional });

            try
            {
                AddHTTPURLReservation(bindingAddress);

                using(HttpSelfHostServer server = new HttpSelfHostServer(configuration))
                {
                    server.OpenAsync().Wait();

                    Console.WriteLine("서버를 중단하기 위해서 아무 키나 눌러 주시기 바랍니다.");

                    Console.ReadKey(true);
                }
            }
            finally
            {
                DeleteHTTLURLReservation(bindingAddress);
            }
        }

        #endregion

        #region HTTP URL 예약 추가하기 - AddHTTPURLReservation(bindingAddress)

        /// <summary>
        /// HTTP URL 예약 추가하기
        /// </summary>
        /// <param name="bindingAddress">바인딩 주소</param>
        private static void AddHTTPURLReservation(string bindingAddress)
        {
            try
            {
                string argument = string.Format("http add urlacl url={0} user=everyone listen=yes", bindingAddress);

                ProcessStartInfo processStartInfo = new ProcessStartInfo("netsh", argument);

                processStartInfo.Verb            = "runas";
                processStartInfo.CreateNoWindow  = true;
                processStartInfo.WindowStyle     = ProcessWindowStyle.Hidden;
                processStartInfo.UseShellExecute = true;

                Process.Start(processStartInfo).WaitForExit();
            }
            catch
            {
            }
        }

        #endregion
        #region HTTP URL 예약 삭제하기 - DeleteHTTLURLReservation(bindingAddress)

        /// <summary>
        /// HTTP URL 예약 삭제하기
        /// </summary>
        /// <param name="bindingAddress">바인딩 주소</param>
        private static void DeleteHTTLURLReservation(string bindingAddress)
        {
            try
            {
                string argument = string.Format("http delete urlacl url={0}", bindingAddress);

                ProcessStartInfo processStartInfo = new ProcessStartInfo("netsh", argument);

                processStartInfo.Verb            = "runas";
                processStartInfo.CreateNoWindow  = true;
                processStartInfo.WindowStyle     = ProcessWindowStyle.Hidden;
                processStartInfo.UseShellExecute = true;

                Process.Start(processStartInfo).WaitForExit();
            }
            catch
            {
            }
        }

        #endregion
    }
}